macro(append_subdir_files src_var tgt_var dirname)
get_directory_property(holder DIRECTORY ${dirname} DEFINITION ${src_var})
foreach(depfile ${holder})
   list(APPEND ${tgt_var} "${dirname}/${depfile}")
endforeach()
endmacro()

macro(process_subdir dirname)
    add_subdirectory(${dirname})
    append_subdir_files(SLOFILES ALL_SLO ${dirname})
    append_subdir_files(EXCEPTIONSFILES ALL_EXC ${dirname})
endmacro()

macro(process_idl_subdir dirname)
    add_subdirectory(${dirname})
    append_subdir_files(TARGET DB_NAMES ${dirname})
endmacro()

macro(mkdir dirname)
    execute_process(COMMAND mkdir ${dirname})
endmacro()

# Every exe and lib is tagged with one of these. Eventually install
# commands and such are added to them.
macro(public_library_hook libname)
endmacro()

macro(internal_library_hook libname)
endmacro()

macro(public_executable_hook libname)
endmacro()

macro(internal_executable_hook libname)
endmacro()


# Do not check for dependencies in precompiled header dirs.
include_regular_expression("^([^p]|p[^r]|pr[^e]|pre[^c]|prec[^o]|preco[^m]|precom[^p])")

# Call this like this: add_idl_db(cssbeans "UCR" file1.idl file2.idl file3.idl)
function(add_idl_db dbname merge_key_name)
    set(db_filename "${dbname}.db")
    set(db_dependency_name "${idl-dep-prefix}dephook-${dbname}")
    foreach(idl_filename ${ARGN})
        string(REGEX REPLACE ".idl$" ".urd" urd_filename ${idl_filename})
        add_custom_command(OUTPUT ${urd_filename}
        COMMAND idlc ${IDLCFLAGS} -O${CMAKE_CURRENT_BINARY_DIR} ${CMAKE_CURRENT_SOURCE_DIR}/${idl_filename}
        DEPENDS idlc ${idl_filename})
        list(APPEND urd_files ${urd_filename})
    endforeach()

    add_custom_command(OUTPUT ${db_filename}
    COMMAND regmerge ${CMAKE_CURRENT_BINARY_DIR}/${db_filename} ${merge_key_name} ${urd_files}
    DEPENDS regmerge ${urd_files})

    add_custom_target("${db_dependency_name}" ALL DEPENDS "${db_filename}")
endfunction()

function(build_rdb_from_db rdbname merge_key_name)
    set(rdb_filename "${rdbname}.rdb")
    set(rdb_dependency_name "dephook-${rdbname}")
    #set(db_filelist "")
    foreach(db_filename ${ARGN})
        set(db_filename "${db_filename}.db")
        list(APPEND db_filelist "${CMAKE_CURRENT_BINARY_DIR}/${db_filename}")
    endforeach()

    add_custom_command(OUTPUT ${rdb_filename}
    COMMAND regmerge ${CMAKE_CURRENT_BINARY_DIR}/${rdb_filename} ${merge_key_name} ${db_filelist}
    # Build Java files next.
    COMMAND cppumaker ${CPPUMAKERFLAGS} ${rdb_filename}
    COMMAND javamaker ${JAVAMAKERFLAGS} ${rdb_filename}
    DEPENDS regmerge javamaker cppumaker ${db_filelist})

    add_custom_target("${rdb_dependency_name}" ALL DEPENDS "${rdb_filename}")
endfunction()
 
